@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package JSON::Schema::Tiny; # git description: v0.012-2-g9d89474
+package JSON::Schema::Tiny; # git description: v0.013-5-gb4e2ba8
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate data against a schema, minimally
 # KEYWORDS: JSON Schema data validation structure specification tiny
 
-our $VERSION = '0.013';
+our $VERSION = '0.014';
 
 use 5.020;  # for unicode_strings, signatures, postderef features
 use experimental qw(signatures postderef);
@@ -407,20 +407,22 @@ sub _eval_keyword_type ($data, $schema, $state) {
     }
     abort($state, '"type" values are not unique') if not is_elements_unique($schema->{type});
 
+    my $type = get_type($data);
     return 1 if any {
-      is_type($_, $data)
-        or ($_ eq 'boolean' and $SCALARREF_BOOLEANS and is_type('reference to SCALAR', $data))
+      $type eq $_ or ($_ eq 'number' and $type eq 'integer')
+        or ($_ eq 'boolean' and $SCALARREF_BOOLEANS and $type eq 'reference to SCALAR')
     } $schema->{type}->@*;
-    return E($state, 'wrong type (expected one of %s)', join(', ', $schema->{type}->@*));
+    return E($state, 'got %s, not one of %s', $type, join(', ', $schema->{type}->@*));
   }
   else {
     assert_keyword_type($state, $schema, 'string');
     abort($state, 'unrecognized type "%s"', $schema->{type}//'<null>')
       if not any { ($schema->{type}//'') eq $_ } qw(null boolean object array string number integer);
 
-    return 1 if is_type($schema->{type}, $data)
-      or ($schema->{type} eq 'boolean' and $SCALARREF_BOOLEANS and is_type('reference to SCALAR', $data));
-    return E($state, 'wrong type (expected %s)', $schema->{type});
+    my $type = get_type($data);
+    return 1 if $type eq $schema->{type} or ($schema->{type} eq 'number' and $type eq 'integer')
+      or ($schema->{type} eq 'boolean' and $SCALARREF_BOOLEANS and $type eq 'reference to SCALAR');
+    return E($state, 'got %s, not %s', $type, $schema->{type});
   }
 }
 
@@ -1065,20 +1067,20 @@ sub is_type ($type, $value) {
   return ref($value) eq $type;
 }
 
-# only the core six types are reported (integers are numbers)
-# use is_type('integer') to differentiate numbers from integers.
 sub get_type ($value) {
   return 'null' if not defined $value;
   return 'object' if is_plain_hashref($value);
   return 'array' if is_plain_arrayref($value);
   return 'boolean' if is_bool($value);
 
-  return ref($value) =~ /^Math::Big(?:Int|Float)$/ ? 'number' : (blessed($value) ? '' : 'reference to ').ref($value)
+  return ref($value) =~ /^Math::Big(?:Int|Float)$/ ? ($value->is_int ? 'integer' : 'number')
+      : (blessed($value) ? '' : 'reference to ').ref($value)
     if is_ref($value);
 
   my $flags = B::svref_2object(\$value)->FLAGS;
   return 'string' if $flags & B::SVf_POK && !($flags & (B::SVf_IOK | B::SVf_NOK));
-  return 'number' if !($flags & B::SVf_POK) && ($flags & (B::SVf_IOK | B::SVf_NOK));
+  return int($value) == $value ? 'integer' : 'number'
+    if !($flags & B::SVf_POK) && ($flags & (B::SVf_IOK | B::SVf_NOK));
 
   croak sprintf('ambiguous type for %s',
     JSON::MaybeXS->new(allow_nonref => 1, canonical => 1, utf8 => 0, allow_bignum => 1, allow_blessed => 1)->encode($value));
@@ -1100,7 +1102,7 @@ sub is_equal ($x, $y, $state = {}) {
   return 0 if $types[0] ne $types[1];
   return 1 if $types[0] eq 'null';
   return $x eq $y if $types[0] eq 'string';
-  return $x == $y if $types[0] eq 'boolean' or $types[0] eq 'number';
+  return $x == $y if grep $types[0] eq $_, qw(boolean number integer);
 
   my $path = $state->{path};
   if ($types[0] eq 'object') {
@@ -1268,7 +1270,7 @@ JSON::Schema::Tiny - Validate data against a schema, minimally
 
 =head1 VERSION
 
-version 0.013
+version 0.014
 
 =head1 SYNOPSIS
 
@@ -1502,8 +1504,6 @@ L<Understanding JSON Schema|https://json-schema.org/understanding-json-schema>:
 
 =back
 
-=head1 SUPPORT
-
 =for stopwords OpenAPI
 
 You can also find me on the L<JSON Schema Slack server|https://json-schema.slack.com> and L<OpenAPI Slack
